#!/usr/bin/perl
##############################################################################
#
#  ETS -- Wolfenstein: Enemy Territory Statistics
#  (c) <cade@bis.bg> 2003
#
#  Developed for play.evrocom.net server ET community
#  http://play.evrocom.net
#
#  This software is distributed under GPL license, see COPYING for details.
#
##############################################################################
use strict;

my %gun;
my %kil;
my %ded;
my %wep;
my %wep2; # weapon usage by clans
my %wep3; # weapon usage by players
my %nam;
my %use;  # weapon usage
my %kdr;  # kill/death ratio
my %kdr2; # kill/death ratio for clans

my %noise_p; # noise in words for players :)
my %noise_c; # noise in words for clans :)

my $href = 0;
my %href;

my $floor = 256;

while(<>)
  {
  s/^(\d+):(\d+)\s*//; # get time
  my $time = $1 * 60 + $2;
  s/\^.//g; # clear colors etc.
  
  if( /Kill: (\d+) (\d+) (\d+): (.+) killed (.+) by MOD_([A-Z_]+)/i )
    {
    $gun{ $3 } |= $6;
    
    die "mismatched gun name: $6" if $gun{ $3 } ne $6;
    $kil{ $4 }++;
    $ded{ $5 }++;
    $wep{ $6 }++;
    $kil{ get_clan( $4 ) }++;
    $ded{ get_clan( $5 ) }++;
    
    # weapon usage
    $wep2{ get_clan( $4 ) }{ $6 }++;
    $wep3{ $4 }{ $6 }++;
    
    # get names...
    $nam{ $4 }++;
    $nam{ $5 }++;
    $nam{ get_clan( $4 ) }++;
    $nam{ get_clan( $5 ) }++;

    print STDERR "unknown: $_\n" if $6 eq '';
    next;
    }
  if( /^say(team)?: ([^:]+): (.+)/i )
    {
    my $p = $2;
    my $cnt = $3 =~ /\S+/g;
    $noise_p{ $p } += $cnt;
    $noise_c{ get_clan( $p ) } += $cnt;
    }
  }

$href{ $_ } = $href++ for keys %nam;

my $time = scalar localtime();
print " <html>
        <title>Wolfenstein: Enemy Territory Statistics</title>
        <style>
        <!--
        BODY  { margin: 0px 0px 0px 0px; background-color: #333; color: #fff; }
        TABLE { font-family: monospace; background-color: #888; }
        TR.hh { border: solid 1px #aaa; background-color: #777; font-weight: bold; }
        TR.r1 { border: solid 1px #aaa; background-color: #444; }
        TR.r2 { border: solid 1px #aaa; background-color: #555; }
        A     { font-family: monospace; color: #fff; }
        -->
        </style>
        <center>
        <h1>Wolfenstein: Enemy Territory Statistics</h1>
        <p>
        |
        <a href=#players>players</a> |
        <a href=#clans>clans</a> |
        <a href=#kdr_p>players k/d ratio</a> |
        <a href=#kdr_c>clans k/d ratio</a> |
        <a href=#noise_p>players noise</a> |
        <a href=#noise_c>clans noise</a> |
        <a href=#wep>weapons</a> |
        <p>
        Floor is $floor frags, players below this are not shown (but still counted).
        <p>
        ";

print "<p><a name=players><a href=#top>[top]</a>";
print mod_player_stats();
print "<p><a name=clans><a href=#top>[top]</a>";
print mod_clan_stats();

print "<p><a name=kdr_p><a href=#top>[top]</a>";
print mod_generic_stats( "kill/death ratio by players", \%kdr );
print "<p><a name=kdr_c><a href=#top>[top]</a>";
print mod_generic_stats( "kill/death ratio by clans", \%kdr2 );

print "<p><a name=noise_p><a href=#top>[top]</a>";
print mod_generic_stats( "noise in words by player", \%noise_p );
print "<p><a name=noise_c><a href=#top>[top]</a>";
print mod_generic_stats( "noise in words by clan", \%noise_c );

print "<p><a name=wep><a href=#top>[top]</a>";
print mod_generic_stats( "preferred weapons", \%wep );
for my $k ( sort { $kil{ $b } <=> $kil{ $a } } keys %wep2 )
  {
  print "<p><a name=$href{$k}><a href=#top>[top]</a>";
  print mod_generic_stats( "preferred weapons by clan $k", $wep2{ $k } );
  }
for my $k ( sort { $kil{ $b } <=> $kil{ $a } } keys %wep3 )
  {
  next unless $kil{ $k } > $floor;
  print "<p><a name=$href{$k}><a href=#top>[top]</a>";
  print mod_generic_stats( "preferred weapons by player $k", $wep3{ $k } );
  }
print "<p>
       Thanks to all players at <a href=http://play.evrocom.net>http://play.evrocom.net</a>!<br>
       <i><b>ETS</b> by (c) <a href=mailto:cade\@bis.bg>&lt;cade\@bis.bg&gt;</a> 2003</i>
       </center>
       </html>";

##############################################################################

sub mod_player_stats
{
  return mod_player_clan_stats(0);
}

sub mod_clan_stats
{
  return mod_player_clan_stats(1);
}

sub mod_player_clan_stats
{
  my $clan = shift;

  my $text;
  
  $text .= "<p>";
  $text .= $clan ? "<h2>Clans</h2>" : "<h2>Players</h2>";
  $text .= "<table width=80% cellspacing=1 cellpadding=5>\n";
  $text .= "<tr class=hh>
               <td width=05% align=right>#</td>
               <td width=15%>Clan</td>
               <td width=30%>Player</td>
               <td width=10% align=right>Kills</td>
               <td width=10% align=right>Deaths</td>
               <td width=10% align=right>K/D</td>
               <td width=20%>Prefered Weapon</td>
            </tr>";
  my $row = 0;
  my $sum_k = 0;
  my $sum_d = 0;
  for my $p ( sort { $kil{ $b } <=> $kil{ $a } } keys %nam )
    {
    next if $clan and $p !~ /^!\[/;
    next if !$clan and $p =~ /^!\[/;

    my $k = $kil{ $p } || 0;
    my $d = $ded{ $p } || 0;
    my $r = $d > 0 ? sprintf "%.2f", $k / $d : 'n/a';
    my $w;
    
    ( $w, undef ) = hash_max( $wep2{ $p } ) if $clan;
    ( $w, undef ) = hash_max( $wep3{ $p } ) if !$clan;
    
    $kdr{ $p } = $r if !$clan;
    $kdr2{ $p } = $r if $clan;
    
    my $c = get_clan( $p );
    
    my $target1 = $href{ $c };
    my $target2 = $href{ $p };
    
    $p =~ s/>/&gt;/g;
    $p =~ s/</&lt;/g;
    
    $sum_k += $k;          
    $sum_d += $d;          
    
    next unless $k > $floor;
    
    $row++;
    my $class = tr_class();
    $text .= "<tr class=$class>
                 <td align=right>$row</td>
                 <td><a href=#$target1>$c</a></td>
                 <td><a href=#$target2>$p</a></td>
                 <td align=right>$k</td>
                 <td align=right>$d</td>
                 <td align=right>$r</td>
                 <td><a href=#wep>$w</a></td>
              </tr>";
    }
  $text .= "<tr class=hh>
               <td width=05% align=right></td>
               <td width=15%></td>
               <td width=30%></td>
               <td width=10% align=right>$sum_k</td>
               <td width=10% align=right>$sum_d</td>
               <td width=10% align=right></td>
               <td width=20%></td>
            </tr>";
  $text .= "</table>\n";
  $text .= "\n\n\n\n";
  return $text;
}

sub mod_generic_stats
{
  my $title = shift;
  my $hr    = shift;
  my $text;
  
  my $sum = hash_sum( $hr );
  
  $text .= "<p>";
  $text .= "<h2>$title</h2>";
  $text .= "<table width=80% cellspacing=1 cellpadding=5>\n";
  $text .= "<tr class=hh>
               <td width=05% align=right>#</td>
               <td width=40%>Name</td>
               <td width=30% align=right>Value</td>
               <td width=25% align=right>Percentage</td>
            </tr>";
  my $row = 0;
  for my $k ( sort { $hr->{ $b } <=> $hr->{ $a } } keys %$hr )
    {
    my $v = $hr->{ $k };
    my $p = $sum > 0 ? sprintf "%.2f%%", 100 * $v / $sum : 'n/a';
    
    $row++;
    my $class = tr_class();
    $text .= "<tr class=$class>
                 <td align=right>$row</td>
                 <td>$k</td>
                 <td align=right>$v</td>
                 <td align=right>$p</td>
              </tr>";
    }
  $text .= "</table>\n";
  $text .= "\n\n\n\n";
  return $text;
}

##############################################################################

my $static_tr;

sub tr_class
{
  $static_tr = !$static_tr;
  $static_tr ? 'r1' : 'r2';
}

sub hash_max
{
  my $hr = shift;
  my $max_k = undef;
  my $max_v = undef;
  while( my ( $k, $v ) = each %$hr )
    {
    if( $max_v eq '' or $v > $max_v )
      {
      $max_k = $k;
      $max_v = $v;
      next;
      }
    }
  #print STDERR "$max_k, $max_v\n";
  return ( $max_k, $max_v );
}

sub hash_sum
{
  my $hr = shift;
  my $sum = 0;
  $sum += $_ for values %$hr;
  return $sum;
}

sub get_clan
{
  my $s = shift;
  my $c = '![]';
  $c = "![$1]" if $s =~ /^\[([^\]]+)\]/;
  $c = "![$1]" if $s =~ /^([A-Z]{2,5})-/i;
  return uc $c;
}

###EOF########################################################################

